/* $Id: k8100grab.c,v 0.7 1999/5/4 23:35:33 root Exp root $
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.

 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.

 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 *  k8100grab.c: A video grabber for the k8100 Velleman kit
 *
 *  Written by ps@telin.rug.ac.be (Philippe Serbruyns)
 *
 */

/* define here what the IO address your K8100 Velleman kit has 
 * see it matches the jumper settings 
 */
#define IO_ADDR 0x300		/* 0x31c on my system */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <asm/io.h>
#include <getopt.h>
#include <vga.h>
#include <vgagl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>

extern int ioperm (unsigned long from, unsigned long num, int turn_on);

#define FRAME 512*512		/* frame size */


void
allocerr (char *what, int size)
{
  fprintf (stderr, "Can't allocate %d bytes for %s variable.\n", size, what);
  exit (1);
}				/* Allocation error */

void
startgrab (long delay)
{
  outb (1, IO_ADDR);
  usleep (delay);
}				/* Ignition for the K8100 */

void
main (int argc, char *argv[])
{
  unsigned int i, j;
/* default values for frames is 10 and delay is 80ms */
  int VGAMODE, video_r = 2, small = 0, fd = 0, mkfile = 0,
    pgm= 0, xoffset, yoffset;
  unsigned long delay = 80000L, x, frames = 10L;
  char *header = "P5\n512 512\n255\n\0";
  char *buf, opt, *fp = NULL, file[512];

  GraphicsContext physicalscreen;

  while ((opt = getopt (argc, argv, "f:d:v:sw:hp")) != EOF)
    switch (opt)
      {
      case 'f':
	frames = atol (optarg);
	break;
      case 'd':
	delay = atol (optarg) * 1000L;
	break;
      case 'v':
	video_r = atoi (optarg);
	break;
      case 's':
	small = 1;
	header = "P5\n256 256\n255\n\0";
	break;
      case 'w':
	fp = optarg;
	mkfile = 1;
	if (strstr (fp, "%")==NULL) frames=1; /* output 1 frame */
	break;
      case 'p':
	pgm = 1;
	break;
      case 'h':		/* help */
      default:
	fprintf (stderr,
 "Usage: %s [-f #number_of_frames(10)] [-d #delay_in_milliseconds(80)]\n" \
 "       [-v 0|1|2 (video_resolution)] [-p (output=PGM) ]\n" \
 "       [-s (small)] [-w file%%04d.pgm]\n",
		 argv[0]);
	exit (1);
      }
  if ((buf = malloc (FRAME)) == NULL)
    allocerr ("buf", FRAME);

  ioperm (IO_ADDR, 1L, 1);	/* release IO_ADDR */

/* svgalib function init */
  vga_init ();
  switch (video_r)
    {
    case 0:
      VGAMODE = G320x240x256;
      xoffset = small * 32;
      yoffset = 0;
      break;
    case 1:
      VGAMODE = G640x480x256;
      xoffset = 64 + small * 128;
      yoffset = small * 112;
      break;
    default:
      VGAMODE = G800x600x256;
      xoffset = 144 + small * 128;
      yoffset = 44 + small * 128;
      break;
    }
  if (!vga_hasmode (VGAMODE))
    {
      fprintf (stderr, "Mode not available.\n");
      exit (1);
    }
  vga_setmode (VGAMODE);
  gl_setcontextvga (VGAMODE);	/* Physical screen context. */
  gl_getcontext (&physicalscreen);
  gl_setrgbpalette ();
  gl_clearscreen (0);
  for (i = 0; i < 256; i++)
    vga_setpalette (i, i / 4, i / 4, i / 4);	/* Setup 64 gray levels */

  for (x = 0; x < frames; x++)
    {
      if (mkfile)
	{
	  sprintf (file, fp, x);
	  if ((fd = open (file, O_WRONLY | O_CREAT | O_TRUNC, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH)) < 0)
	    {
	      fprintf (stderr, "Error opening file \"%s\"\n", file);
	      exit (1);
	    }
	  if (pgm) write(fd, header, strlen(header));
	}

      startgrab (delay);

      if (small)
	{			/* Use 256x256 image for faster grab and less space */
	  for (j = 0; j < 256; j++)
	    for (i = 0; i < 256; i++)
	      buf[i + j * 256] = (inb (IO_ADDR) + inb (IO_ADDR)) / 2;
	  for (j = 0; j < 256; j++)
	    {
	      vga_drawscansegment (buf + j * 256, xoffset, j + yoffset, 256);
	      if (mkfile)
		write (fd, buf + j * 256, 256);
	    }
	}
      else
	{
	  for (j = 0; j < 512; j++)
	    for (i = 0; i < 512; i++)
	      buf[i + j * 512] = inb (IO_ADDR);
	  for (j = 0; j < 512; j++)
	    if (j & 1)
	      {
		vga_drawscansegment (buf + FRAME / 2 + (j - 1) * 256, xoffset, j + yoffset, 512);
		if (mkfile)
		  write (fd, buf + FRAME / 2 + (j - 1) * 256, 512);
	      }
	    else
	      {
		vga_drawscansegment (buf + j * 256, xoffset, j + yoffset, 512);
		if (mkfile)
		  write (fd, buf + j * 256, 512);
	      }
	}
      if (mkfile)
	close (fd);
    }
  vga_setmode (TEXT);
  exit (0);
}
